<?php

namespace Repository;

use Util\InstanceProvider;
use Zeedhi\DevOps\Util\Utilities;
use Teknisa\Model\Traits\TQuery;

class CpCall
{
    use TQuery;

    public function getEntityManager(){
        return InstanceProvider::getEntityManager();
    }

    public function findAllCalls($customQuery, $orderBy, $page, $pageSize)
    {
        $startPos = ($page - 1) * $pageSize;
        $finalPos = $page * $pageSize;
        $query = <<<SQL
            SELECT   COUNT(1) OVER () TOTAL_LINES_ZHNEXT,CALL.NMOPERRESP, CALL.ID, CALL.NRORG, CALL.CDCLIENTE, CALL.CDFILIAL, CALL.CALL_RELATIONSHIP, CALL.TEKNISA_ID,
                   CALL.REFERENCE, CALL.CLASSIFICATION, CALL.CLASSIFICATION_NAME, CALL.REPORTED_BY, CALL.IS_ACTIVE,
                   CALL.REPORTER_NAME, CALL.PRODUCT, CALL.CAUSE, CALL.CAUSE_NAME, CALL.PRODUCT_NAME, CALL.CREATED_ORDER,
                   CALL.UPDATED, TO_CHAR(CALL.CREATED_AT, 'DD/MM/YYYY') AS CREATED_AT, CALL.CLOSED, CALL.STATUS,
                   CALL.PLANNED_DATE, CALL.UPDATED_ORDER, TO_CHAR(CALL.UPDATED_AT, 'DD/MM/YYYY') AS UPDATED_AT, TO_CHAR(CALL.APPOINTMENT_DATE_INIT, 'DD/MM/YYYY') AS APPOINTMENT_DATE_INIT,
                    TO_CHAR(CALL.APPOINTMENT_DATE, 'DD/MM/YYYY') AS APPOINTMENT_DATE,
                   CALL.MAIN_TYPE_CALL, CALL.MAIN_TYPE_CALL_DESCRIPTION, CALL.TYPE, CALL.VERSION_DEV,
                   (SELECT RTRIM(XMLAGG(XMLELEMENT(E, LABEL.DESCRIPTION, ',')
                                          .EXTRACT('//text()')
                                          ORDER BY LABEL.DESCRIPTION
                                       ).GETCLOBVAL(), ','
                                ) AS LABEL_DESCRIPTION
                      FROM CP_CALL_LABEL_RELATION LABEL_REL
                      JOIN CP_CALL_LABEL LABEL ON LABEL_REL.LABEL = LABEL.NRCALLLABEL
                     WHERE LABEL_REL.CALL = CALL.ID
                       ) AS LABEL_DESCRIPTION,
                   CALL.NMRAZSOCCLIE, CALL.NMFANTCLIE, CALL.TEKNISA_INTERLOCUTOR_CODE, CALL.PLANNED_VERSION,
                   CALL.PLANNED_VALUE, CALL.RESPONSABLE_DIVISION_CODE, CALL.CUSTOMER_INTERLOCUTOR_CODE,
                   CALL.CUSTOMER_RESPONSABLE_CODE, CALL.CUSTOMER_INTERLOCUTOR, CALL.CALL_PRIORITY, CALL.RESPONSIBLE_OF_THE_PRODUCT,
                   CALL.RESPONSIBLE_OF_THE_CUSTOMER, CALL.TEKNISA_INTERLOCUTOR, CALL.RESPONSABLE_DIVISION,
                   CALL.TEXT_SMALL_1, CALL.TEXT_SMALL_2, CALL.TEXT_SMALL_3, CALL.TEXT_SMALL_4, CALL.TEXT_SMALL_5,
                   CALL.TEXT_LONG_1,CALL.TEXT_LONG_2,CALL.TEXT_LONG_3,CALL.TEXT_LONG_4,CALL.TEXT_LONG_5,
                   CALL.NUMBER_1,CALL.NUMBER_2,CALL.NUMBER_3,CALL.NUMBER_4,CALL.NUMBER_5,CALL.
                   DATE_1,CALL.DATE_2,CALL.DATE_3,CALL.DATE_4,CALL.DATE_5,CALL.PRODUCT_TYPE_NAME,FORWARD_CALL_GRID
              FROM ($customQuery) CALL
         LEFT JOIN CP_CALL_LABEL_RELATION LABEL_REL ON LABEL_REL.CALL = CALL.ID
         LEFT JOIN CP_CALL_LABEL LABEL ON LABEL_REL.LABEL = LABEL.NRCALLLABEL
          GROUP BY CALL.NMOPERRESP, CALL.ID, CALL.NRORG, CALL.CDCLIENTE, CALL.CDFILIAL, CALL.CALL_RELATIONSHIP, CALL.TEKNISA_ID,
                   CALL.REFERENCE, CALL.CLASSIFICATION, CALL.CLASSIFICATION_NAME, CALL.REPORTED_BY, CALL.IS_ACTIVE,
                   CALL.REPORTER_NAME, CALL.PRODUCT, CALL.CAUSE, CALL.CAUSE_NAME, CALL.PRODUCT_NAME, CALL.CREATED_ORDER,
                   CALL.UPDATED, CALL.CREATED_AT, CALL.CLOSED, CALL.STATUS, CALL.PLANNED_DATE, CALL.UPDATED_ORDER,
                   CALL.UPDATED_AT, CALL.MAIN_TYPE_CALL, CALL.MAIN_TYPE_CALL_DESCRIPTION, CALL.TYPE, CALL.NMRAZSOCCLIE,
                   CALL.NMFANTCLIE, CALL.TEKNISA_INTERLOCUTOR_CODE, CALL.PLANNED_VERSION, CALL.PLANNED_VALUE,
                   CALL.RESPONSABLE_DIVISION_CODE, CALL.CUSTOMER_INTERLOCUTOR_CODE, CALL.CUSTOMER_RESPONSABLE_CODE,
                   CALL.CUSTOMER_INTERLOCUTOR, CALL.CALL_PRIORITY, CALL.RESPONSIBLE_OF_THE_PRODUCT,
                   CALL.RESPONSIBLE_OF_THE_CUSTOMER, CALL.TEKNISA_INTERLOCUTOR, CALL.RESPONSABLE_DIVISION, CALL.VERSION_DEV,
                   CALL.TEXT_SMALL_1, CALL.TEXT_SMALL_2, CALL.TEXT_SMALL_3, CALL.TEXT_SMALL_4, CALL.TEXT_SMALL_5,
                   CALL.TEXT_LONG_1,CALL.TEXT_LONG_2,CALL.TEXT_LONG_3,CALL.TEXT_LONG_4,CALL.TEXT_LONG_5,
                   CALL.NUMBER_1,CALL.NUMBER_2,CALL.NUMBER_3,CALL.NUMBER_4,CALL.NUMBER_5,CALL.DATE_1,
                   CALL.DATE_2,CALL.DATE_3,CALL.DATE_4,CALL.DATE_5,CALL.PRODUCT_TYPE_NAME,FORWARD_CALL_GRID,CALL.APPOINTMENT_DATE_INIT,
                   CALL.APPOINTMENT_DATE
                   ORDER BY $orderBy
SQL;
        //TODO buga o ordenar por data
        //return $this->createAndExecuteQueryAndBuildResponseData(InstanceProvider::getEntityManager(), $query);
        $query = Utilities::getPaginatedQuery($query,$finalPos,$startPos);
        $result =  $this->getEntityManager()->getConnection()->fetchAll($query);
        $total = empty($result) ? 0 : $result[0]['TOTAL_LINES_ZHNEXT'];
        $returnNext = [ 'data' => $result, 'pagination' => [ 'limit' => $pageSize, 'page' => $page, 'total' => $total]];
        return $returnNext;
    }

    public function findSoliservById($soliservId, $situationFilters, $userId)
    {
        $gepOpened  = $situationFilters['GEP']['OPENED'];
        $gepWaiting = $situationFilters['GEP']['WAITING'];
        $gepClosed  = $situationFilters['GEP']['CLOSED'];
        $validate = \Util\Utilities::getValidateBudgetFaseGep();
        $refuse = \Util\Utilities::getRefuseBudgetFaseGep();

        $query = <<<SQL
        SELECT  SER.NRSOLISERV AS ID, SER.NRORG, SER.DSREFESOLI AS REFERENCE,
                SER.CDOPERINCLUSAO AS CREATED_BY, SER.NRLICENCAUSO AS USE_LICENSE,
                DES.DSSOLISERV AS DESCRIPTION, OPE.CDOPERADOR AS REPORTER_BY,CLI.CDCLIENTE ,
                TO_CHAR((SELECT MAX(LFT.DTINCLUSAO) FROM LOGFASETRAM LFT WHERE (LFT.NRSOLISERV = :SOLISERV_ID AND LFT.CDFILIAL = SER.CDFILIAL) ), 'DD/MM/YYYY') AS UPDATED_AT,
                SER.CDPRODCOMP AS PRODUCT, 'S' AS TYPE, OPE.NMOPERADOR AS REPORTER_NAME,
                '' AS CAUSE, SER.CDOPERULTATU AS UPDATED, PRO.CDPRODUTO AS BOARD,
                SER.CDPRODGRUP AS PRODUCT_TYPE, PRT.NMPRODUTO AS PRODUCT_TYPE_NAME, CLI.NMRAZSOCCLIE,
                ASS.NMASSUNTO AS CLASSIFICATION_NAME, ASS.IDORIGOBRSOL AS CLASSIFICATION_IDORIGOBRSOL,
                PRO.NMPRODUTO AS PRODUCT_NAME, ASS.CDASSUNTO AS CLASSIFICATION,
                 CASE WHEN SER.CDFASETRAM IN ('$validate')
                       THEN 'Y'
                       ELSE 'N'
                     END AS ENABLE_APPROVAL,
                 CASE WHEN EXISTS (   SELECT NRSEQORCAMENTO AS EXTERNAL_ID
                                        FROM FCT_ORCAMENTO
                                       WHERE NRSOLISERV = SER.NRSOLISERV
                                         AND IDATIVO = 'S'
                                         AND IDSITUACAOORCAMENTO <> 'CANCELADO')
                       THEN 'Y'
                       ELSE 'N'
                 END AS IS_INTEGRATION_SM,
                CASE WHEN $gepWaiting AND SER.CDFASETRAM NOT IN (SELECT CDFATRAMENCE FROM PARAMGERAL PARA WHERE PARA.NRORG = SER.NRORG)
                     THEN 'Y'
                     ELSE 'N'
                 END AS ENABLE_CLOSED,
                CASE WHEN $gepClosed
                     THEN 'Fechado'
                     WHEN $gepWaiting
                     THEN 'Aguardando ação do cliente'
                     WHEN $gepOpened
                     THEN 'Em aberto'
                     ELSE 'Em progresso'
                END AS STATUS,
                CASE WHEN $gepWaiting
                     THEN 'Y'
                     ELSE 'N'
                 END AS WAITING,
                (SELECT S.NRSOLISERV
                   FROM SOLISERV S
                   JOIN ORCASOLICITA O ON (O.CDFILIAL = S.CDFILIAL AND O.NRSOLISERV = S.NRSOLISERV)
                  WHERE S.NRSOLISERV = :SOLISERV_ID
                    AND ROWNUM = 1
                    AND  (SELECT CDFASETRAM
                                   FROM ( SELECT CDFASETRAM
                                            FROM LOGFASETRAM LFT
                                           WHERE LFT.NRSOLISERV = :SOLISERV_ID
                                             AND CDFASETRAM IN ('$refuse','$validate')
                                        ORDER BY DTALFASETRAM DESC)
                                  WHERE ROWNUM = 1) IN ('$validate')) AS WITH_BUDGET,
                SER.CDVERPROORIG AS PRODUCT_VERSION, SER.CDVERPROORIG AS PRODUCT_VERSION_LABEL,
                SER.CDRELVERORIG AS PRODUCT_RELEASE, SER.CDRELVERORIG AS PRODUCT_RELEASE_LABEL,
                SER.CDPATRELORIG AS PRODUCT_PATCH, SER.CDPATRELORIG AS PRODUCT_PATCH_LABEL
         FROM  SOLISERV SER
         JOIN  DESCSOLI DES ON DES.NRSOLISERV = SER.NRSOLISERV
         JOIN  OPERADOR OPE ON OPE.CDOPERADOR = SER.CDOPERADOR
         JOIN  PRODUTO PRO  ON PRO.CDPRODUTO = SER.CDPRODCOMP
    LEFT JOIN  PRODUTO PRT  ON PRT.CDPRODUTO = SER.CDPRODGRUP
         JOIN  ASSUNTO ASS  ON ASS.CDASSUNTO = SER.CDASSUNTO
    LEFT JOIN CLIENTE CLI ON CLI.CDCLIENTE = SER.CDCLIENTE
        WHERE  SER.NRSOLISERV = :SOLISERV_ID
SQL;

        $params = array(
            "SOLISERV_ID" => $soliservId,
            "USER_ID" => $userId
        );
        $result = InstanceProvider::getEntityManager()->getConnection()->fetchAll($query, $params);

        return count($result) > 0 ? $result[0] : array();
    }

    public function findIssueById($issueId, $situationFilters = null)
    {
        $gepWaiting = '0';
        if($situationFilters){
            $gepWaiting = implode($situationFilters['PM']['WAITING'],',');
        }
        $query = <<<SQL
              SELECT  TAS.ID, TAS.ORGANIZATION AS NRORG, TAS.NAME AS REFERENCE,
                      TAS.CREATED_BY, TAS.DESCRIPTION, OPR.CDOPERADOR AS REPORTER_BY,CLI.CDCLIENTE ,
                      TO_CHAR(TAS.UPDATED_AT, 'DD/MM/YYYY') AS UPDATED_AT,
                      TAS.PROJECT, TAS.PROJECT AS PRODUCT, OPR.NMOPERADOR AS REPORTER_NAME,
                      TAS.ID AS CALL_RELATIONSHIP, '' AS CAUSE, TAS.UPDATED_BY AS CDOPERULTATU, CLI.NMRAZSOCCLIE,
                      TAS.STATUS,TAS.STATUS AS STATUS,PRO.ID AS BOARD,
                      PRO.NAME AS PRODUCT_NAME, CLA.ID AS CLASSIFICATION, CLA.LABEL AS CLASSIFICATION_NAME,
                      DECODE(PVD.ID, NULL, 'Última versão', PVD.VERSION || '.' || PVD.REVISION) AS VERSION_DEV,
                      CASE WHEN TS.CONTROL = 'H'
                          THEN 'Y'
                          ELSE 'N'
                      END AS ENABLE_CLOSED,
                        CASE WHEN TS.ID IN ($gepWaiting)
                             THEN 'Y'
                             ELSE 'N'
                         END AS WAITING,
                      CASE WHEN TS.CONTROL = 'A'
                             THEN 'Y'
                             ELSE 'N'
                      END AS ENABLE_APPROVAL,
                      CASE WHEN EXISTS (SELECT EXTERNAL_ID FROM BUDGET WHERE TASK = :ISSUE_ID AND EXTERNAL_ID IS NOT NULL)
                           THEN 'Y'
                           ELSE 'N'
                      END AS IS_INTEGRATION_SM,
                      (SELECT CONTROL
                         FROM (SELECT CTY.CONTROL
                                 FROM COMMENTS COM
                                 JOIN COMMENT_TYPE CTY ON(COM.COMMENT_TYPE_ID = CTY.ID )
                                WHERE COM.TASK = TAS.ID
                             ORDER BY COM.ID DESC)
                        WHERE ROWNUM = 1 ) AS LAST_COMMENT_TYPE,
                      CASE WHEN EXISTS( SELECT ID FROM TASK_PROPERTIES TPR WHERE TPR.FINAL_TASK_STATUS = TS.ID)
                              THEN 'Y'
                              ELSE null
                      END AS CLOSED
                FROM TASK TAS
                JOIN TASK_STATUS TS ON (TS.ID = TAS.TASK_STATUS)
                JOIN PROJECT PRO ON (PRO.ID = TAS.PROJECT)
                JOIN CLASSIFICATION CLA ON (CLA.ID = TAS.CLASSIFICATION)
                JOIN RESOURCE_T RT ON RT.ID = TAS.CREATED_BY
           LEFT JOIN PRODUCT_VERSION PVD ON PVD.ID = TAS.PRODUCT_VERSION_ID_DEV
           LEFT JOIN ALLOCATION SA ON (SA.TASK = TAS.ID AND SA.TYPE = 'S')
           LEFT JOIN RESOURCE_T SRE ON (SRE.ID = SA.RESOURCE_T)
           LEFT JOIN CLIENTE CLI ON CLI.CDCLIENTE = SRE.EXTERNAL_CODE
           LEFT JOIN ALLOCATION RA ON (RA.TASK = TAS.ID AND RA.TYPE = 'R')
           LEFT JOIN RESOURCE_T RRE ON (RRE.ID = RA.RESOURCE_T)
           LEFT JOIN OPERADOR OPR ON OPR.CDOPERADOR = RRE.EXTERNAL_CODE
               WHERE TAS.ID = :ISSUE_ID
SQL;

        $params = array(
            "ISSUE_ID" => $issueId
        );

        $result =  $this->getEntityManager()->getConnection()->fetchAll($query, $params);
        return count($result)>0 ? $result[0] : array();
    }

    public function findCallTypeIssue($id, $situationFilters){


        $pmWaiting = implode($situationFilters['PM']['WAITING'],',');
        $query = <<<SQL
              SELECT  C.NRCALL AS ID, C.NRORG, C.REFERENCE, C.DESCRIPTION, OPR.CDOPERADOR AS REPORTER_BY, OPR.CDOPERADOR AS REPORTED_BY,CLI.CDCLIENTE, 
                        TO_CHAR(C.DTCONPINIT, 'DD/MM/YYYY') AS APPOINTMENT_DATE_INIT,TO_CHAR(C.DTCONPCUREN, 'DD/MM/YYYY') AS APPOINTMENT_DATE,
                          TO_CHAR(C.DTULTATU, 'DD/MM/YYYY') AS UPDATED_AT, C.PRODUCT, C.TYPE,OPR.NMOPERADOR AS REPORTER_NAME,
                          C.CALL_RELATIONSHIP, C.CAUSE, C.CDOPERULTATU, C.IDATIVO,C.IDATIVO AS STATUS,NVL(C.PRODUCT, PRO.ID) AS BOARD, CLI.NMRAZSOCCLIE,
                          PRO.NAME AS PRODUCT_NAME,
                          CLA.ID AS CLASSIFICATION,CLA.LABEL AS CLASSIFICATION_NAME,
                            CASE WHEN TS.CONTROL = 'H'
                                 THEN 'Y'
                                 ELSE 'N'
                             END AS ENABLE_CLOSED,
                            CASE WHEN TS.CONTROL = 'A'
                                 THEN 'Y'
                                 ELSE 'N'
                             END AS ENABLE_APPROVAL,
                            CASE WHEN EXISTS (SELECT EXTERNAL_ID FROM BUDGET WHERE TASK = TAS.ID AND EXTERNAL_ID IS NOT NULL)
                                 THEN 'Y'
                                 ELSE 'N'
                            END AS IS_INTEGRATION_SM,
                            CASE WHEN TS.ID IN ($pmWaiting) OR (  SELECT CONTROL FROM (SELECT CTY.CONTROL
                                                                    FROM COMMENTS COM
                                                                    JOIN COMMENT_TYPE CTY ON(COM.COMMENT_TYPE_ID = CTY.ID )
                                                                    WHERE COM.TASK = TAS.ID
                                                                    ORDER BY COM.ID DESC)
                                                                    WHERE ROWNUM = 1 ) = 'Q'
                                 THEN 'Y'
                                 ELSE null
                             END AS WAITING,
                          (
                             SELECT rtrim(xmlagg(xmlelement(e,CLR.LABEL ,',').extract('//text()') order by CLR.LABEL ).GetClobVal(),',')
                             FROM   CP_CALL_LABEL_RELATION CLR
                             WHERE  CLR.CALL = C.NRCALL
                             AND    CLR.NRORG = C.NRORG
                          ) AS CALL_LABELS,
                         (SELECT CONTROL FROM (SELECT CTY.CONTROL
                                                FROM COMMENTS COM
                                                JOIN COMMENT_TYPE CTY ON(COM.COMMENT_TYPE_ID = CTY.ID )
                                                WHERE COM.TASK = TAS.ID
                                                ORDER BY COM.ID DESC)
                                                WHERE ROWNUM = 1 ) AS LAST_COMMENT_TYPE,
                         PRV.ID AS MENU_VERSION_ID,
                         DECODE(PRV.ID,NULL,NULL,(PRV.VERSION || '.' || PRV.REVISION  || '.' || PRV.BUILD)) AS MENU_VERSION,
                         DECODE(PVD.ID, NULL, 'Última versão', PVD.VERSION || '.' || PVD.REVISION) AS VERSION_DEV,
                         DECODE(PVD.ID, NULL, 'Última versão', PVD.ID) AS VERSION_DEV_ID,
                         OPR.LABEL AS MENU_ITEM ,OPR.ID AS MENU_ITEM_ID,
                         OPR.NREXTERNAL_PRODUCT_ID AS DEVOPS_ID,
                         NVL(PRD.ID,PDP.ID) AS DEVOPS_ID_PRODUCT,C.TEXT_SMALL_1, C.TEXT_SMALL_2, C.TEXT_SMALL_3, C.TEXT_SMALL_4, C.TEXT_SMALL_5,
                         C.TEXT_LONG_1,C.TEXT_LONG_2,C.TEXT_LONG_3,C.TEXT_LONG_4,C.TEXT_LONG_5,C.NUMBER_1,C.NUMBER_2,C.NUMBER_3,C.NUMBER_4,C.NUMBER_5,
                         TO_CHAR(DATE_1, 'DD/MM/YYYY') AS DATE_1,TO_CHAR(DATE_2, 'DD/MM/YYYY') AS DATE_2,TO_CHAR(DATE_3, 'DD/MM/YYYY') AS DATE_3,TO_CHAR(DATE_4, 'DD/MM/YYYY') AS DATE_4,TO_CHAR(DATE_5, 'DD/MM/YYYY') AS DATE_5
                FROM CP_CALL C
                JOIN OPERADOR OPE ON (OPE.CDOPERADOR = C.REPORTED_BY )
                JOIN TASK TAS ON(TAS.ID = C.CALL_RELATIONSHIP)
                JOIN TASK_STATUS TS ON (TS.ID = TAS.TASK_STATUS)
                JOIN PROJECT PRO ON(PRO.ID = NVL(C.PRODUCT, TAS.PROJECT))
           LEFT JOIN PROJECT PRC ON(PRC.ID = C.PRODUCT)
           LEFT JOIN PRODUCT PRD ON(PRD.PROJECT_ID = PRC.ID)
           LEFT JOIN PRODUCT PDP ON(PDP.PROJECT_ID = PRO.ID)
           LEFT JOIN PRODUCT_VERSION PRV ON (TAS.PRODUCT_VERSION_ID = PRV.ID)
           LEFT JOIN OM_PRODUTO_ELEMENTO OPR On (TAS.MENU_ITEM_ID = OPR.ID)
           LEFT JOIN PRODUCT_VERSION PVD ON PVD.ID = TAS.PRODUCT_VERSION_ID_DEV
                JOIN CLASSIFICATION CLA ON (CLA.ID = TAS.CLASSIFICATION)
           LEFT JOIN ALLOCATION SA ON (SA.TASK = TAS.ID AND SA.TYPE = 'S')
           LEFT JOIN RESOURCE_T SRE ON (SRE.ID = SA.RESOURCE_T)
           LEFT JOIN CLIENTE CLI ON CLI.CDCLIENTE = SRE.EXTERNAL_CODE
           LEFT JOIN ALLOCATION RA ON (RA.TASK = TAS.ID AND RA.TYPE = 'R')
           LEFT JOIN RESOURCE_T RRE ON (RRE.ID = RA.RESOURCE_T)
           LEFT JOIN OPERADOR OPR ON OPR.CDOPERADOR = RRE.EXTERNAL_CODE
               WHERE C.NRCALL = :CALL_ID
SQL;

        $params = array(
            "CALL_ID" => $id
        );

        $result =  $this->getEntityManager()->getConnection()->fetchAll($query, $params);
        return count($result)>0 ? $result[0] : array();
    }

    /**
     * @param $cdCliente
     * @param $callType
     * @param $callRelationship
     * @param $cdprodcomp
     * @return array
     */
    public function findProductCallProductsNew($cdCliente,  $callType, $callRelationship, $cdprodcomp) {
        $query =
            <<<SQL
            SELECT * FROM (
            SELECT PRD.NMPRODUTO, 'I' AS CALL_TYPE, 'MOBILE' AS PLATAFORMA, 'Zeedhi: App, Browser, Mobile' AS PLATAFORMA_NAME,
                   PDO.PROJECT_ID, NULL AS SGSISTEMA, TO_CHAR(PDO.PROJECT_ID) AS PRODUCT_FILTER_ID
              FROM OM_PRODUTO_ORG PRO
              JOIN OM_PRODUTO PRD ON PRD.ID = PRO.NRPRODUTO_ID
              JOIN PRODUCT PDO    ON PDO.ID = PRD.NREXTERNAL_PRODUCT_ID
              JOIN ORGANIZACAO ORG ON ORG.NRORG = PRO.NRORG_ID
              JOIN CLIENTE CLI ON CLI.NRPARCNEGOCIO = ORG.NRPARCNEGOCIO
             WHERE CLI.CDCLIENTE = :CDCLIENTE

               AND NOT EXISTS (SELECT ID
                                 FROM OM_PRODUTO_ELEMENTO PEL
                                WHERE PEL.NRPRODUTO_ID = PRD.ID
                                  AND PEL.CDSISTEMA IS NOT NULL)
         UNION ALL
        SELECT DISTINCT SIS.NMSISTEMA, 'S' AS CALL_TYPE, 'DELPHI' AS PLATAFORMA,
                        'Delphi: Windows' AS PLATAFORMA_NAME, NULL AS PROJECT_ID,
                        SIS.SGSISTEMA, NVL(PRC.CDPRODUTO, SIS.SGSISTEMA) AS PRODUCT_FILTER_ID
                  FROM ORGANIZACAO ORG
                  JOIN CLIENTE CLI      ON CLI.NRPARCNEGOCIO = ORG.NRPARCNEGOCIO
                  JOIN CONTRATOLINP CLP ON CLP.CDCLIENTE = CLI.CDCLIENTE
                                       AND CLP.DTCONTLINPRESC IS NULL
                  JOIN SISTCONTLINP SCL ON SCL.CDCLIENTE = CLP.CDCLIENTE
                                       AND SCL.CDLINPCLIE = CLP.CDLINPCLIE
                  JOIN SISTEMA SIS      ON SIS.CDSISTEMA = SCL.CDSISTEMA
             LEFT JOIN SOLISERV SOL ON SOL.NRSOLISERV = :NRSOLISERV
                  LEFT JOIN PRODUTO PRG ON PRG.CDPRODUTO = SOL.CDPRODGRUP
                  LEFT JOIN PRODUTO PRC ON PRC.CDARVPROD = PRG.CDARVPROD||SIS.SGSISTEMA
                 WHERE CLI.CDCLIENTE = :CDCLIENTE
                   AND NVL(:CALL_TYPE, '-') IN ('-', 'S')
                   AND NVL(PRC.CDPRODUTO, '-') = NVL(:CDPRODCOMP, NVL(PRC.CDPRODUTO, '-'))
              ORDER BY 2 DESC, 1)
        WHERE 1=1

SQL;

        $params = array(
            "CDCLIENTE" => $cdCliente,
            "NRSOLISERV" => $callRelationship,
            "CALL_TYPE" => $callType,
            "CDPRODCOMP" => $cdprodcomp
        );
        return $this->createAndExecuteQueryAndBuildResponseData(InstanceProvider::getEntityManager(), $query, $params);
    }

    public function getClassificationSoli() {
        $query = <<<SQL
        	SELECT ASS.CDASSUNTO AS ID, CLA.LABEL AS LABEL, ASS.IDORIGOBRSOL
            FROM CLASSIFICATION CLA
            JOIN ASSUNTO ASS ON (CLA.CDASSUNTO = ASS.CDASSUNTO)
           
SQL;
        return $this->createAndExecuteQueryAndBuildResponseData(InstanceProvider::getEntityManager(), $query);
    }

    public function findProductCallClassificationIssue($nrOrg) {

        $query = <<<SQL
        SELECT CLA.ID, CLA.LABEL,DECODE(CLA.CDASSUNTO,NULL,'Zeedhi','Zeedhi e Delphi') AS PLATAFORMA_NAME
          FROM CLASSIFICATION CLA
          JOIN CLASSIFICATION_PROJECT_TYPE CPT ON (CPT.CLASSIFICATION = CLA.ID AND CPT.TYPE = 'P')
          JOIN PROJECT_TYPE PRT ON (PRT.ID = CPT.PROJECT_TYPE)
         WHERE PRT.EXTERNALTYPE = 'P'
           AND CLA.IS_TASK = 'Y'
           AND CLA.CONTROL IS NULL
           AND CLA.ORGANIZATION = :ORGANIZATION_ID
      GROUP BY CLA.LABEL, CLA.ID, CLA.CDASSUNTO
      ORDER BY CLA.LABEL
SQL;

        $params = array(
            'ORGANIZATION_ID' => $nrOrg,
        );
        return $this->createAndExecuteQueryAndBuildResponseData(InstanceProvider::getEntityManager(), $query, $params);
    }

    public function findProductCallProductTypes($productCode = '')
    {

        $query = <<<SQL
             SELECT *
              FROM( SELECT P.CDPRODUTO as ID, P.NMPRODUTO AS LABEL, p.CDARVPROD
                      FROM PRODUTO P, VERSAOPROD V
                     WHERE (P.CDNVPRODUTO = '3')
                       AND (NOT P.CDARVPROD LIKE 'P4%')
                       AND (
                               (P.NMPRODUTO LIKE '%Sistemas%Vers%')
                                     OR (UPPER(P.NMPRODUTO) LIKE '%SITE%JAVA%')
                                     OR (UPPER(P.NMPRODUTO) LIKE 'PROD%SETFORMS%VERS%')
                                     OR (P.CDPRODUTO IN ('0000001386', '0000004180'))
                                    AND NOT P.NMPRODUTO LIKE '%Componente%'
                           )
                       AND P.CDPRODUTO = V.CDPRODGRUP(+)
                       AND NVL(V.IDVERSAOATIV, 'N') = 'S'

                    ORDER BY 2 DESC
                  )
                WHERE 1=1
                and id in   (  SELECT * FROM (
            SELECT
               DISTINCT GRUP.cdproduto as id
            FROM
               PARFILIALGEP PG, PRSUBGRPROD NV, PRODUTO PR,
               (SELECT CDARVPROD, CDPRODUTO FROM PRODUTO WHERE CDPRODUTO = CDPRODUTO) GRUP
            WHERE
                (PG.CDFILIAL = '9999')
                AND (NV.CDFILISUBGR = '9999' OR NV.CDFILISUBGR = 'P')
                AND (PG.CDGRUPPROD = NV.CDGRUPPROD)
                AND (NV.CDPRODUTO = PR.CDPRODUTO)
                AND (PR.CDARVPROD LIKE GRUP.CDARVPROD||'%')
                AND pr.CDARVPROD like '%$productCode'
        ))            
        
SQL;
        return $this->createAndExecuteQueryAndBuildResponseData(InstanceProvider::getEntityManager(), $query);
    }

    public function getVersao($P_CDGRUPSIST, $P_CDSISTEMA, $P_CDVERSGSIS = '', $P_DSVERSGSIS = '', $PRODUCT_FILTER = false)
    {

        $adqPesqVersao = <<<SQL
            SELECT  PR.CDVERPRODUTO AS ID, PR.CDVERPRODUTO AS LABEL , CDPRODCOMP
                FROM PRODLIBEVERS PR WHERE (PR.CDPRODGRUP = :P_CDPRODGRUP )
                AND (PR.CDPRODCOMP in  ( SELECT
               DISTINCT PR.CDPRODUTO AS ID
            FROM
               PARFILIALGEP PG, PRSUBGRPROD NV, PRODUTO PR,
               (SELECT CDARVPROD, CDPRODUTO FROM PRODUTO WHERE CDPRODUTO = NVL(:P_CDPRODGRUP,CDPRODUTO)) GRUP
            WHERE
                (PG.CDFILIAL = '9999')
                AND (NV.CDFILISUBGR = '9999' OR NV.CDFILISUBGR = 'P')
                AND (PG.CDGRUPPROD = NV.CDGRUPPROD)
                AND (NV.CDPRODUTO = PR.CDPRODUTO)
                AND (PR.CDARVPROD LIKE GRUP.CDARVPROD||'%')
                AND pr.CDARVPROD = :P_SISTID))
SQL;
        $adqPesqVersaoParams = array(
            'P_CDPRODGRUP' => $P_CDGRUPSIST,
            'P_CDPRODCOMP' => $P_CDSISTEMA,
            'P_SISTID' => $PRODUCT_FILTER,
            'P_CDVERPRODUTO' => ''
        );

        return $this->createAndExecuteQueryAndBuildResponseData(InstanceProvider::getEntityManager(), $adqPesqVersao,$adqPesqVersaoParams);
    }

    public function findAllOperadorResponsible(){
        $query  = <<<SQL
        SELECT CDOPERADOR, NMOPERADOR, IDATIVO, NMRAZSOCCLIE, CDCLIENTE
        FROM (SELECT OP.CDOPERADOR, OP.NMOPERADOR, OP.IDATIVO, CL.NMRAZSOCCLIE, OP.CDCLIENTE
                      FROM OPERADOR OP
                      JOIN CLIENTE CL ON (CL.CDCLIENTE = OP.CDCLIENTE)
                      AND OP.IDOPERATIVO = 'A'
                      AND CL.IDATIVO = 'S'
        )
        ORDER BY NMOPERADOR

SQL;
        return $this->createAndExecuteQueryAndBuildResponseData(InstanceProvider::getEntityManager(), $query);
    }

    public function getReleases($productTypeId, $version) {
        $query =
            <<<SQL
            SELECT CDRELVERPROD AS ID, CDRELVERPROD AS LABEL
            FROM RELEVERPROD
            WHERE
            (CDPRODGRUP = :P_CDGRUPSIST) AND
            (CDVERPRODUTO = :P_CDVERSGSIS)
            ORDER BY CDRELVERPROD DESC
SQL;

        $params = array(
            "P_CDGRUPSIST" => $productTypeId,
            "P_CDVERSGSIS" => $version,
        );

        return $this->createAndExecuteQueryAndBuildResponseData(InstanceProvider::getEntityManager(), $query, $params);
    }

    public function getPatch($productType, $version, $release) {
        $query =
            <<<SQL
            SELECT CDPATCHVERPR AS ID,CDPATCHVERPR AS LABEL
              FROM PATCHVERPROD
             WHERE CDPRODGRUP = :P_CDGRUPSIST
               AND CDVERPRODUTO = :P_CDVERSGSIS
               AND CDRELVERPROD = :P_CDRELEASEVER
          ORDER BY CDPATCHVERPR DESC
SQL;

        $params = array(
            "P_CDGRUPSIST" => $productType,
            "P_CDVERSGSIS" => $version,
            "P_CDRELEASEVER" => $release,
        );

        return $this->createAndExecuteQueryAndBuildResponseData(InstanceProvider::getEntityManager(), $query, $params);
    }

    public function findAllCustomers() {
        $query =
            <<<SQL
            SELECT CDCLIENTE, NMRAZSOCCLIE, NMFANTCLIE, NRORG FROM CLIENTE WHERE IDATIVO = 'S'
SQL;

        return $this->createAndExecuteQueryAndBuildResponseData(InstanceProvider::getEntityManager(), $query);
    }

    public function findProductCallUseLicense($cdCliente)
    {
        $query =
            <<<SQL
           SELECT NRLICENCAUSO as ID, NRLICENCAUSO || ' - ' || NMFANTLUSO as LABEL
              FROM LICENCAUSO
             WHERE ((NRLICENCAUSO = :P_NRLICENCAUSO) OR ('T' = :P_TNRLICENCAUSO))
               AND (CDCLIENTE = :P_CDCLIENTE)
          ORDER BY NRLICENCAUSO
        
SQL;

        $params = array(
            'P_TNRLICENCAUSO' => "T",
            'P_NRLICENCAUSO' => "",
            'P_CDCLIENTE' => $cdCliente,
        );

        return $this->createAndExecuteQueryAndBuildResponseData(InstanceProvider::getEntityManager(), $query, $params);
    }

    public function getAllCallLabels() {
        $query = <<<SQL
            SELECT C.NRCALLLABEL AS ID, C.DESCRIPTION,
                   C.CDOPERINCLUSAO AS CREATED_BY, O.NMOPERADOR AS CREATED_BY_NAME,
                   C.CDCLIENTE, CL. NMFANTCLIE, CL.NMRAZSOCCLIE
              FROM CP_CALL_LABEL C
         LEFT JOIN OPERADOR O ON O.CDOPERADOR = C.CDOPERINCLUSAO
         LEFT JOIN CLIENTE CL ON CL.CDCLIENTE = C.CDCLIENTE
             
SQL;
        return $this->createAndExecuteQueryAndBuildResponseData(InstanceProvider::getEntityManager(), $query);
    }

    public function findMenuItems ($pmProductId, $cdCliente) {
        $return = array();
        $productDevOpsId = InstanceProvider::getRepository()->getCpCall()->getProductDevOpsIdByProjectId($pmProductId);
        if($productDevOpsId) {
            $return = $this->getProductElementsByProductIdAndCdcliente($productDevOpsId, $cdCliente);
        }
        return $return;
    }

    public function getProductElementsByProductIdAndCdcliente($productId, $cdCliente) {
        $query = <<<SQL
        SELECT OP.NREXTERNAL_PRODUCT_ID, OP.ID, OP.LABEL, OP.ID AS "id", OP.PARENT_ID AS "parent_id", OP.LABEL AS "label"
          FROM CLIENTE C, ORGANIZACAO O, OM_PRODUTO_ORG OO, OM_PRODUTO_ORG_ELEMENTO OE, OM_PRODUTO_ELEMENTO OP
         WHERE C.CDCLIENTE = :CDCLIENTE
           AND O.NRPARCNEGOCIO = C.NRPARCNEGOCIO
           AND OO.NRORG_ID = O.NRORG
           AND OO.NRPRODUTO_ID = (SELECT MAX(ID) FROM OM_PRODUTO OD WHERE OD.NREXTERNAL_PRODUCT_ID = :PRODUCT_ID)
           AND OE.NRPRODUTO_ORG_ID = OO.ID
           AND OP.NRPRODUTO_ID = OO.NRPRODUTO_ID
           AND OP.ID = OE.NRPRODUTO_ELEMENTO_ID
           AND OP.LABEL IS NOT NULL
SQL;
        $params = array(
            'PRODUCT_ID' => $productId,
            'CDCLIENTE'  => $cdCliente
        );

        return $this->createAndExecuteQueryAndBuildResponseData(InstanceProvider::getEntityManager(), $query,$params);
    }

    public function findProductVersionWithConcatVersionByProduct($productId,$organizationId) {
        $query = <<<SQL
        SELECT PR.PRODUCT_ID, PR.ID, (PR.VERSION || '.' || PR.REVISION || '.' || PR.BUILD) AS VERSION
          FROM PRODUCT_VERSION PR
         WHERE PR.PRODUCT_ID = :PRODUCT
           AND PR.ORGANIZATION_ID = :ORGANIZATION
      ORDER BY PR.VERSION DESC, PR.REVISION DESC, PR.BUILD DESC
SQL;
        $params = array(
            'PRODUCT' => $productId,
            'ORGANIZATION' => $organizationId,
        );

        return $this->createAndExecuteQueryAndBuildResponseData(InstanceProvider::getEntityManager(), $query,$params);
    }

    public function getQueryIntranetCalls($situationFilters, $specialFilters, $situation, $whereFilters, $clienteWhere, $filterSLA = '',$filterSLAClas = '',$filterSLAAss = '', $soliFilter = '')
    {
        $gepOpened  = $situationFilters['GEP']['OPENED'];
        $gepWaiting = $situationFilters['GEP']['WAITING'];
        $gepClosed  = $situationFilters['GEP']['CLOSED'];

        $cdClienteTeknsia = \Util\Utilities::getCdClienteBase();

        $query = <<<SQL
            SELECT * FROM(SELECT  TO_CHAR(ID) AS ID,
                    TO_CHAR(NRORG) AS NRORG,
                    TO_CHAR(NRSOLISERV) AS CALL_RELATIONSHIP,
                    TO_CHAR(CDCLIENTE) AS CDCLIENTE,
                    TO_CHAR(CDFILIAL) AS CDFILIAL,
                    TO_CHAR(REFERENCE) AS REFERENCE,
                    TO_CHAR(CLASSIFICATION) AS CLASSIFICATION,
                    TO_CHAR(CLASSIFICATION_NAME) AS CLASSIFICATION_NAME,
                    TO_CHAR(REPORTED_BY) AS REPORTED_BY,
                    TO_CHAR(IS_ACTIVE) AS IS_ACTIVE,
                    TO_CHAR(REPORTER_NAME) AS REPORTER_NAME,
                    TO_CHAR(PRODUCT) AS PRODUCT,
                    TO_CHAR(TYPE) AS TYPE,
                    LABEL_DESCRIPTION, NRCALLLABEL,
                        CASE TYPE
                        WHEN 'I'
                            THEN 'I - ' || CALL_RELATIONSHIP
                        WHEN 'S'
                            THEN 'S - ' || CALL_RELATIONSHIP
                        ELSE
                            CALL_RELATIONSHIP
                        END AS TEKNISA_ID,
                    TO_CHAR(CAUSE) AS CAUSE,
                    TO_CHAR(CAUSE_NAME) AS CAUSE_NAME,
                    TO_CHAR(PRODUCT_NAME) AS PRODUCT_NAME,
                    TO_CHAR(CREATED_ORDER) AS CREATED_ORDER,
                    TO_CHAR(NVL(UPDATED_AT,UPDATED), 'DD/MM/YYYY') AS UPDATED,
                    CREATED_AT,
                    TO_CHAR(CLOSED) AS CLOSED,
                    TO_CHAR(STATUS) AS STATUS,
                    TO_CHAR(UPDATED_AT, 'YYYYMMDD') AS UPDATED_ORDER,
                    UPDATED_AT,                    
                    APPOINTMENT_DATE_INIT,
                    APPOINTMENT_DATE,
                    TO_CHAR(DTCOMPROMISS) AS PLANNED_DATE,
                    'P' AS MAIN_TYPE_CALL,
                    'Produto' AS MAIN_TYPE_CALL_DESCRIPTION,
                    TO_CHAR(PRODUCT_RESPONSABLE_CODE) AS PRODUCT_RESPONSABLE_CODE,
                    TO_CHAR(TEKNISA_INTERLOCUTOR_CODE) AS TEKNISA_INTERLOCUTOR_CODE,
                    TO_CHAR(PLANNED_VERSION) AS PLANNED_VERSION,
                    TO_CHAR(PLANNED_VALUE) AS PLANNED_VALUE,
                    TO_CHAR(RESPONSABLE_DIVISION_CODE) AS RESPONSABLE_DIVISION_CODE,
                    TO_CHAR(CUSTOMER_RESPONSABLE_CODE) AS CUSTOMER_RESPONSABLE_CODE,
                    TO_CHAR(CUSTOMER_INTERLOCUTOR_CODE) AS CUSTOMER_INTERLOCUTOR_CODE,
                    TO_CHAR(CUSTOMER_INTERLOCUTOR) AS CUSTOMER_INTERLOCUTOR,
                    TO_CHAR(CALL_PRIORITY) AS CALL_PRIORITY,
                    TO_CHAR(RESPONSIBLE_OF_THE_PRODUCT) AS RESPONSIBLE_OF_THE_PRODUCT,
                    TO_CHAR(RESPONSIBLE_OF_THE_CUSTOMER) AS RESPONSIBLE_OF_THE_CUSTOMER,
                    TO_CHAR(TEKNISA_INTERLOCUTOR) AS TEKNISA_INTERLOCUTOR,
                    TO_CHAR(RESPONSABLE_DIVISION) AS RESPONSABLE_DIVISION,
                    TO_CHAR(NMRAZSOCCLIE) AS NMRAZSOCCLIE,
                    TO_CHAR(NMFANTCLIE) AS NMFANTCLIE,
                    TO_CHAR(null) AS VERSION_DEV,
                    TO_CHAR(CDOPERRESP) AS CDOPERRESP,
                    TO_CHAR(NMOPERRESP) AS NMOPERRESP,TEXT_SMALL_1, TEXT_SMALL_2, TEXT_SMALL_3, TEXT_SMALL_4, TEXT_SMALL_5,
                    TEXT_LONG_1,TEXT_LONG_2,TEXT_LONG_3,TEXT_LONG_4,TEXT_LONG_5,
                    NUMBER_1,NUMBER_2,NUMBER_3,NUMBER_4,NUMBER_5,DATE_1,DATE_2,DATE_3,DATE_4,DATE_5,
                    TO_CHAR(PRODUCT_TYPE_NAME) AS PRODUCT_TYPE_NAME,
                    TO_CHAR(CDARVPROD) AS CDARVPROD,FORWARD_CALL_GRID
                 FROM  (
                    SELECT  SER.NRSOLISERV,C.NRCALL AS ID, NVL(C.NRORG, SER.NRORG) AS NRORG, SER.CDCLIENTE, SER.CDFILIAL,
                            SER.DSREFESOLI AS REFERENCE, NVL(CLA.ID,ASS.CDASSUNTO) AS CLASSIFICATION,
                            SER.CDOPERADOR AS REPORTED_BY, '' AS CAUSE_NAME,
                            SER.IDATIVO AS IS_ACTIVE, OPE.NMOPERADOR AS REPORTER_NAME,
                            SER.CDPRODCOMP AS PRODUCT, NVL(C.TYPE, 'S') AS TYPE,
                            PRO.NMPRODUTO AS PRODUCT_NAME, NVL(CLA.LABEL,ASS.NMASSUNTO) AS CLASSIFICATION_NAME,
                            TO_CHAR(SER.NRSOLISERV) AS CALL_RELATIONSHIP, C.CAUSE, SER.DTULTATU AS UPDATED, SER.DTCOMPROMISS,
                            TO_CHAR(SER.DTINCLUSAO, 'YYYYMMDD') AS CREATED_ORDER,
                            SER.DTINCLUSAO AS CREATED_AT,
                            C.PRODUCT_RESPONSABLE_CODE,
                            C.TEKNISA_INTERLOCUTOR_CODE,
                            C.PLANNED_VERSION,
                            C.PLANNED_VALUE,
                            C.RESPONSABLE_DIVISION_CODE,
                            C.CUSTOMER_INTERLOCUTOR_CODE,
                            C.CUSTOMER_RESPONSABLE_CODE,
                            INTE.NMINTECLIE AS CUSTOMER_INTERLOCUTOR,
                            PRI.DSPRIORIDADE AS CALL_PRIORITY,
                            RESPROD.NMOPERADOR AS RESPONSIBLE_OF_THE_PRODUCT,
                            RESPCLIE.NMOPERADOR AS RESPONSIBLE_OF_THE_CUSTOMER,
                            TEKINTE.NMOPERADOR AS TEKNISA_INTERLOCUTOR,
                            CLASOPORT.NMCLASSOPORT AS RESPONSABLE_DIVISION,
                            CLI.NMRAZSOCCLIE,
                            CLI.NMFANTCLIE,
                            LABEL.DESCRIPTION AS LABEL_DESCRIPTION,
                            LABEL.NRCALLLABEL,
                            (SELECT MAX(LFT.DTINCLUSAO) FROM LOGFASETRAM LFT WHERE (LFT.NRSOLISERV = SER.NRSOLISERV AND LFT.CDFILIAL = SER.CDFILIAL) )AS UPDATED_AT,
                            C.DTCONPINIT AS APPOINTMENT_DATE_INIT ,C.DTCONPCUREN AS APPOINTMENT_DATE,
                           CASE WHEN $gepClosed  THEN 'Fechado'
                                WHEN $gepWaiting THEN 'Aguardando ação do cliente'
                                WHEN $gepOpened  THEN 'Em aberto'
                                ELSE 'Em progresso'
                            END AS STATUS,
                           CASE WHEN (SER.CDFASETRAM IN (SELECT CDFATRAMENCE
                                                           FROM PARAMGERAL PARA
                                                          WHERE PARA.NRORG = SER.NRORG))
                                  OR (SER.IDSOLIENCE = 'S')
                                THEN 'Y'
                           ELSE NULL
                            END AS CLOSED,
                            TO_CHAR(CDOPERRESP) AS CDOPERRESP,
                            TO_CHAR(OPER.NMOPERADOR) AS NMOPERRESP, C.TEXT_SMALL_1, C.TEXT_SMALL_2, C.TEXT_SMALL_3, C.TEXT_SMALL_4, C.TEXT_SMALL_5,
                            C.TEXT_LONG_1,C.TEXT_LONG_2,C.TEXT_LONG_3,C.TEXT_LONG_4,C.TEXT_LONG_5,C.NUMBER_1,C.NUMBER_2,C.NUMBER_3,C.NUMBER_4,C.NUMBER_5,
                            C.DATE_1,C.DATE_2,C.DATE_3,C.DATE_4,C.DATE_5,PRT.NMPRODUTO as PRODUCT_TYPE_NAME, PRO.CDARVPROD,
                            CASE WHEN $gepClosed THEN 'N' ELSE 'S' END AS FORWARD_CALL_GRID
                      FROM  SOLISERV SER
                      JOIN  OPERADOR OPE ON OPE.CDOPERADOR = SER.CDOPERADOR
                      JOIN  PRODUTO PRO  ON PRO.CDPRODUTO = SER.CDPRODCOMP
                      JOIN  ASSUNTO ASS  ON ASS.CDASSUNTO = SER.CDASSUNTO
                 LEFT JOIN PRODUTO PRT ON  (PRT.CDPRODUTO = SER.CDPRODGRUP)
                 LEFT JOIN CLASSIFICATION CLA ON(CLA.CDASSUNTO = ASS.CDASSUNTO)
                 LEFT JOIN OPERADOR OPER ON SER.CDOPERRESP = OPER.CDOPERADOR
                 LEFT JOIN  CP_CALL C    ON C.CALL_RELATIONSHIP = SER.NRSOLISERV AND C.TYPE = 'S'
                 LEFT JOIN INTECLIE INTE ON INTE.CDINTECLIE = C.CUSTOMER_INTERLOCUTOR_CODE AND INTE.CDCLIENTE = SER.CDCLIENTE
                 LEFT JOIN CRM_PRIORIDADE PRI ON PRI.NRPRIORIDADE = C.PRIORITY_CODE
                 LEFT JOIN OPERADOR RESPROD ON RESPROD.CDOPERADOR = C.PRODUCT_RESPONSABLE_CODE
                 LEFT JOIN PARCLIEFAS PARCLIE ON PARCLIE.CDCLIENTE = SER.CDCLIENTE AND IDTPFATUCLIE = '5'
                 LEFT JOIN OPERADOR RESPCLIE ON RESPCLIE.CDOPERADOR = PARCLIE.CDGERENTEIMPCLI
                 LEFT JOIN OPERADOR TEKINTE ON TEKINTE.CDOPERADOR = C.TEKNISA_INTERLOCUTOR_CODE
                 LEFT JOIN CRM_CLASSOPORTUNI CLASOPORT ON CLASOPORT.NRSEQCLASSOPORT = C.RESPONSABLE_DIVISION_CODE
                 LEFT JOIN CLIENTE CLI ON CLI.CDCLIENTE = SER.CDCLIENTE
                 LEFT JOIN CP_CALL_LABEL_RELATION LABEL_REL ON LABEL_REL.CALL = C.NRCALL
                 LEFT JOIN CP_CALL_LABEL LABEL ON LABEL_REL.LABEL = LABEL.NRCALLLABEL
                 WHERE ($clienteWhere)
                            $specialFilters
                            $situation
                            $filterSLA
                            $filterSLAClas
                            $filterSLAAss
                            $soliFilter))
                  WHERE 1=1
                        $whereFilters
SQL;
        return $query;
    }

    public function getQueryPmCalls($situationFilters, $specialFilters, $situation, $whereFilters, $clienteWhere,$filterSLA = '',$filterSLAClas = '', $mobFilter = '')
    {
        $pmOpened   = implode($situationFilters['PM']['OPENED'], ',');
        $pmWaiting  = implode($situationFilters['PM']['WAITING'], ',');
        $pmClosed   = implode($situationFilters['PM']['CLOSED'], ',');

        $query = <<<SQL
                    SELECT * FROM(SELECT TO_CHAR(ID) AS ID,
                    TO_CHAR(NRORG) AS NRORG,
                    TO_CHAR(ISSUE_ID) AS CALL_RELATIONSHIP,
                    TO_CHAR(CDCLIENTE) AS CDCLIENTE,
                    TO_CHAR(CDFILIAL) AS CDFILIAL,
                    TO_CHAR(REFERENCE) AS REFERENCE,
                    TO_CHAR(CLASSIFICATION) AS CLASSIFICATION,
                    TO_CHAR(CLASSIFICATION_NAME) AS CLASSIFICATION_NAME,
                    TO_CHAR(REPORTED_BY) AS REPORTED_BY,
                    TO_CHAR(IS_ACTIVE) AS IS_ACTIVE,
                    TO_CHAR(REPORTER_NAME) AS REPORTER_NAME,
                    TO_CHAR(PRODUCT) AS PRODUCT,
                    TO_CHAR(TYPE) AS TYPE,
                    LABEL_DESCRIPTION, NRCALLLABEL,
                    CASE TYPE
                     WHEN 'I'
                         THEN 'I - ' || CALL_RELATIONSHIP
                     WHEN 'S'
                         THEN 'S - ' || CALL_RELATIONSHIP
                     ELSE
                         CALL_RELATIONSHIP
                     END AS TEKNISA_ID,
                    TO_CHAR(CAUSE) AS CAUSE,
                    TO_CHAR(CAUSE_NAME) AS CAUSE_NAME,
                    TO_CHAR(PRODUCT_NAME) AS PRODUCT_NAME,
                    TO_CHAR(CREATED_ORDER) AS CREATED_ORDER,
                    TO_CHAR(NVL(UPDATED_AT, UPDATED), 'DD/MM/YYYY') AS UPDATED,
                    CREATED_AT,
                    TO_CHAR(CLOSED) AS CLOSED,
                    TO_CHAR(STATUS) AS STATUS,
                    TO_CHAR(NVL(UPDATED_AT, UPDATED), 'YYYYMMDD') AS UPDATED_ORDER,
                    UPDATED_AT,
                    APPOINTMENT_DATE_INIT,
                    APPOINTMENT_DATE,
                    TO_CHAR(END_DATE) AS PLANNED_DATE,
                    'P' AS MAIN_TYPE_CALL,
                    'Produto' AS MAIN_TYPE_CALL_DESCRIPTION,
                    TO_CHAR(PRODUCT_RESPONSABLE_CODE) AS PRODUCT_RESPONSABLE_CODE,
                    TO_CHAR(TEKNISA_INTERLOCUTOR_CODE) AS TEKNISA_INTERLOCUTOR_CODE,
                    TO_CHAR(PLANNED_VERSION) AS PLANNED_VERSION,
                    TO_CHAR(PLANNED_VALUE) AS PLANNED_VALUE,
                    TO_CHAR(RESPONSABLE_DIVISION_CODE) AS RESPONSABLE_DIVISION_CODE,
                    TO_CHAR(CUSTOMER_RESPONSABLE_CODE) AS CUSTOMER_RESPONSABLE_CODE,
                    TO_CHAR(CUSTOMER_INTERLOCUTOR_CODE) AS CUSTOMER_INTERLOCUTOR_CODE,
                    TO_CHAR(CUSTOMER_INTERLOCUTOR) AS CUSTOMER_INTERLOCUTOR,
                    TO_CHAR(CALL_PRIORITY) AS CALL_PRIORITY,
                    TO_CHAR(RESPONSIBLE_OF_THE_PRODUCT) AS RESPONSIBLE_OF_THE_PRODUCT,
                    TO_CHAR(RESPONSIBLE_OF_THE_CUSTOMER) AS RESPONSIBLE_OF_THE_CUSTOMER,
                    TO_CHAR(TEKNISA_INTERLOCUTOR) AS TEKNISA_INTERLOCUTOR,
                    TO_CHAR(RESPONSABLE_DIVISION) AS RESPONSABLE_DIVISION,
                    TO_CHAR(NMRAZSOCCLIE) AS NMRAZSOCCLIE,
                    TO_CHAR(NMFANTCLIE) AS NMFANTCLIE,
                    TO_CHAR(VERSION_DEV) AS VERSION_DEV,
                    TO_CHAR(CDOPERRESP) AS CDOPERRESP,
                    TO_CHAR((SELECT RTRIM(XMLAGG(XMLELEMENT(e,RT.NAME ,',').EXTRACT('//text()')).GetClobVal(),',')
                              FROM RESOURCE_PROJECT RP
                              JOIN RESOURCE_T RT ON RT.ID = RP.RESOURCE_C
                             WHERE RP.PROJECT = PRODUCT_ID
                             AND RP.TYPE = 'M')) AS NMOPERRESP,TEXT_SMALL_1, TEXT_SMALL_2, TEXT_SMALL_3, TEXT_SMALL_4, TEXT_SMALL_5,
                             TEXT_LONG_1,TEXT_LONG_2,TEXT_LONG_3,TEXT_LONG_4,TEXT_LONG_5,
                             NUMBER_1,NUMBER_2,NUMBER_3,NUMBER_4,NUMBER_5,DATE_1,DATE_2,DATE_3,DATE_4,DATE_5,
                         'Produto Mobile' AS PRODUCT_TYPE_NAME, null as CDARVPROD,'N' AS FORWARD_CALL_GRID
                  FROM (
                  SELECT TAS.ID AS ISSUE_ID, C.NRCALL AS ID, TAS.ORGANIZATION AS NRORG, SRE.EXTERNAL_CODE AS CDCLIENTE, NULL AS CDFILIAL, TAS.NAME AS REFERENCE,
                         TAS.CLASSIFICATION, OPE.CDOPERADOR AS REPORTED_BY,C.DTCONPINIT AS APPOINTMENT_DATE_INIT,C.DTCONPCUREN AS APPOINTMENT_DATE,
                         TAS.STATUS AS IS_ACTIVE, OPE.NMOPERADOR AS REPORTER_NAME,
                         NVL(C.PRODUCT, TAS.PROJECT) AS PRODUCT, TO_CHAR(TAS.ID) AS CALL_RELATIONSHIP, 'I' AS TYPE,
                         C.CAUSE, TAS.UPDATED_AT AS UPDATED, TAS.END_DATE,
                         TO_CHAR(TAS.CREATED_AT, 'YYYYMMDD') AS CREATED_ORDER,
                         TAS.CREATED_AT, PRO.NAME AS PRODUCT_NAME,
                         CLA.LABEL AS CLASSIFICATION_NAME, '' AS CAUSE_NAME,
                         C.PRODUCT_RESPONSABLE_CODE,
                         C.TEKNISA_INTERLOCUTOR_CODE,
                         C.PLANNED_VERSION,
                         C.PLANNED_VALUE,
                         C.RESPONSABLE_DIVISION_CODE,
                         C.CUSTOMER_INTERLOCUTOR_CODE,
                         C.CUSTOMER_RESPONSABLE_CODE,
                         INTE.NMINTECLIE AS CUSTOMER_INTERLOCUTOR,
                         PRI.DSPRIORIDADE AS CALL_PRIORITY,
                         RESPROD.NMOPERADOR AS RESPONSIBLE_OF_THE_PRODUCT,
                         RESPCLIE.NMOPERADOR AS RESPONSIBLE_OF_THE_CUSTOMER,
                         TEKINTE.NMOPERADOR AS TEKNISA_INTERLOCUTOR,
                         CLASOPORT.NMCLASSOPORT AS RESPONSABLE_DIVISION,
                         CLI.NMRAZSOCCLIE,
                         CLI.NMFANTCLIE,
                         LABEL.DESCRIPTION AS LABEL_DESCRIPTION,
                         LABEL.NRCALLLABEL,
                         DECODE(PVD.ID, NULL, 'Última versão', PVD.VERSION || '.' || PVD.REVISION) AS VERSION_DEV,
                         PVD.VERSION AS VERSION_TESTE,
                         (SELECT MAX(COM.UPDATED_AT) FROM COMMENTS COM WHERE (TAS.ID = COM.TASK) ) AS UPDATED_AT,
                         CASE WHEN EXISTS( SELECT ID FROM TASK_PROPERTIES TPR WHERE TPR.FINAL_TASK_STATUS = TS.ID)
                              THEN 'Y'
                              ELSE null
                         END AS CLOSED,
                         CASE WHEN TS.ID IN($pmWaiting) OR (SELECT NVL(CONTROL,'I') FROM (SELECT CTY.CONTROL
                                                    FROM COMMENTS COM
                                                    left JOIN COMMENT_TYPE CTY ON(COM.COMMENT_TYPE_ID = CTY.ID )
                                                    WHERE COM.TASK = TAS.ID
                                                    ORDER BY COM.ID DESC)
                                                    WHERE ROWNUM = 1 ) = 'Q'
                            THEN 'Aguardando ação do cliente'
                         WHEN TS.ID IN($pmOpened)
                              THEN 'Em aberto'
                         WHEN TS.ID IN($pmClosed)
                             THEN 'Fechado'
                             ELSE 'Em progresso'
                         END AS STATUS,
                         RT.EXTERNAL_CODE AS CDOPERRESP,
                         OPE.NMOPERADOR AS NMOPERRESP,
                         PRC.ID AS PRODUCT_ID, C.TEXT_SMALL_1, C.TEXT_SMALL_2, C.TEXT_SMALL_3, C.TEXT_SMALL_4, C.TEXT_SMALL_5,
                          C.TEXT_LONG_1,C.TEXT_LONG_2,C.TEXT_LONG_3,C.TEXT_LONG_4,C.TEXT_LONG_5,C.NUMBER_1,C.NUMBER_2,C.NUMBER_3,C.NUMBER_4,C.NUMBER_5,
                           C.DATE_1,C.DATE_2,C.DATE_3,C.DATE_4,C.DATE_5, 'Produto Mobile' as PRODUCT_TYPE_NAME, null as CDARVPROD
                    FROM TASK TAS
                    JOIN ALLOCATION SAR ON (SAR.TASK = TAS.ID AND SAR.TYPE = 'R')
                    JOIN RESOURCE_T RES ON RES.ID = SAR.RESOURCE_T
                    JOIN TASK_STATUS TS ON TS.ID = TAS.TASK_STATUS
               LEFT JOIN CP_CALL C      ON TAS.ID = C.CALL_RELATIONSHIP AND C.TYPE = 'I'
               LEFT JOIN OPERADOR OPE   ON OPE.CDOPERADOR = RES.EXTERNAL_CODE
               LEFT JOIN PROJECT PRO    ON (PRO.ID = NVL(C.PRODUCT, TAS.PROJECT))
               LEFT JOIN CLASSIFICATION CLA ON(CLA.ID = TAS.CLASSIFICATION)
               LEFT JOIN ALLOCATION SA ON (SA.TASK = TAS.ID AND SA.TYPE = 'S')
               LEFT JOIN RESOURCE_T SRE ON (SRE.ID = SA.RESOURCE_T)
               LEFT JOIN PROJECT PRC ON(PRC.ID = TAS.PROJECT)
               LEFT JOIN RESOURCE_PROJECT RP ON RP.PROJECT = PRC.ID
               LEFT JOIN RESOURCE_T RT ON RT.ID = RP.RESOURCE_C
               LEFT JOIN INTECLIE INTE ON INTE.CDINTECLIE = C.CUSTOMER_INTERLOCUTOR_CODE AND INTE.CDCLIENTE = SRE.EXTERNAL_CODE
               LEFT JOIN CRM_PRIORIDADE PRI ON PRI.NRPRIORIDADE = C.PRIORITY_CODE
               LEFT JOIN OPERADOR RESPROD ON RESPROD.CDOPERADOR = C.PRODUCT_RESPONSABLE_CODE
               LEFT JOIN PARCLIEFAS PARCLIE ON PARCLIE.CDCLIENTE = SRE.EXTERNAL_CODE AND IDTPFATUCLIE = '5'
               LEFT JOIN OPERADOR RESPCLIE ON RESPCLIE.CDOPERADOR = PARCLIE.CDGERENTEIMPCLI
               LEFT JOIN OPERADOR TEKINTE ON TEKINTE.CDOPERADOR = C.TEKNISA_INTERLOCUTOR_CODE
               LEFT JOIN CRM_CLASSOPORTUNI CLASOPORT ON CLASOPORT.NRSEQCLASSOPORT = C.RESPONSABLE_DIVISION_CODE
               LEFT JOIN CLIENTE CLI ON CLI.CDCLIENTE = SRE.EXTERNAL_CODE
               LEFT JOIN CP_CALL_LABEL_RELATION LABEL_REL ON LABEL_REL.CALL = C.NRCALL
               LEFT JOIN CP_CALL_LABEL LABEL ON LABEL_REL.LABEL = LABEL.NRCALLLABEL
               LEFT JOIN PRODUCT_VERSION PVD ON PVD.ID = TAS.PRODUCT_VERSION_ID_DEV
                   WHERE TAS.IS_ISSUE = 'Y'
                     AND TAS.STATUS NOT IN ('D')
                     AND ($clienteWhere)
                         $specialFilters
                         $situation
                         $filterSLA
                         $filterSLAClas
                         $mobFilter
                         ))
               WHERE 1=1
                     $whereFilters
SQL;
        return $query;
    }

    public function getInfoForSendCall($cdfasetram,$lang,$nrorg) {
        $query =
            <<<SQL
            SELECT DECODE(:P_IDIDIOMA, 'BRA', FAS.TXPERGFATRAM, NVL(PFT.TXPERGFATRAM, FAS.TXPERGFATRAM)) AS TXPERGFATRAM,
                   DECODE(:P_IDIDIOMA, 'BRA', FTD.TXRESPFATRAM, NVL(RFT.TXRESPFATRAM, FTD.TXRESPFATRAM)) AS TXRESPFATRAM,
                   DECODE(:P_IDIDIOMA, 'BRA', FAS.NMFASETRAM, NVL(NFT.NMFASETRAM, FAS.NMFASETRAM)) AS NMFASETRAM,
                   FTD.NRFASTRAMDES,
                   FAS.IDOBSOBRTRAM,
                   FTD.CDFASTRAMDES,
                    CASE WHEN FTD.CDFASTRAMDES IN (SELECT CDFATRAMENCE FROM PARAMGERAL PARA WHERE PARA.NRORG = :NRORG)
                         THEN 'Y'
                         ELSE 'N'
                     END AS IS_FINALIZATION,
                   FTD.TXACAOFATRAM
              FROM FASETRAMDEST FTD, FASETRAM FAS,
                   (SELECT SUBSTR(NMCOMPONENTE,1,3) AS CDFASETRAM, STVALORPROP AS TXPERGFATRAM
                      FROM PROPRIED
                     WHERE CDPROGTRAD = 'FASETRAM'
                       AND SUBSTR(NMCOMPONENTE,1,3) = :P_CDFASE
                       AND IDIDIOMA = :P_IDIDIOMA
                       AND NMPROPRIED = 'TXPERGFATRAM') PFT,
                    (SELECT SUBSTR(NMCOMPONENTE,1,3) AS CDFASETRAM, SUBSTR(NMCOMPONENTE,4,2) AS NRFASTRAMDES, STVALORPROP AS TXRESPFATRAM
                       FROM PROPRIED
                      WHERE CDPROGTRAD = 'FASETRAMDEST'
                        AND SUBSTR(NMCOMPONENTE,1,3) = :P_CDFASE
                        AND IDIDIOMA = :P_IDIDIOMA
                        AND NMPROPRIED = 'TXRESPFATRAM') RFT,
                    (SELECT SUBSTR(NMCOMPONENTE,1,3) AS CDFASETRAM, STVALORPROP AS NMFASETRAM
                       FROM PROPRIED
                      WHERE CDPROGTRAD = 'FASETRAM'
                        AND SUBSTR(NMCOMPONENTE,1,3) = :P_CDFASE
                        AND IDIDIOMA = :P_IDIDIOMA
                        AND NMPROPRIED = 'NMFASETRAM') NFT
             WHERE (FAS.CDFASETRAM = FTD.CDFASETRAM )
               AND (FAS.CDFASETRAM = :P_CDFASE)
               AND (PFT.CDFASETRAM(+) = FAS.CDFASETRAM)
               AND (NFT.CDFASETRAM(+) = FAS.CDFASETRAM)
               AND (RFT.CDFASETRAM(+) = FTD.CDFASETRAM)
               AND (RFT.NRFASTRAMDES(+) = FTD.NRFASTRAMDES)
               ORDER BY FTD.NRFASTRAMDES
SQL;

        $params = array(
            "P_IDIDIOMA" => $lang,
            "P_CDFASE" => $cdfasetram,
            "NRORG" => $nrorg,
        );

        return $this->createAndExecuteQueryAndBuildResponseData(InstanceProvider::getEntityManager(), $query,$params);
    }

    public function operadorByResponseIssue($IssueId)
    {
        $query =
            <<<SQL
            SELECT RTRIM(XMLAGG(XMLELEMENT(e,RT.NAME ,',').EXTRACT('//text()')).GetClobVal(),',') AS NMOPERADOR,
                   '1' AS CDOPERADOR, null AS DSEMAILOPER, '010' ASNRPRIOPERCLI
              FROM RESOURCE_PROJECT RP
              JOIN TASK TAS ON (TAS.ID = :ISSUE_ID AND TAS.PROJECT = RP.PROJECT)
              JOIN RESOURCE_T RT ON (RT.ID = RP.RESOURCE_C AND RP.TYPE = 'M')
SQL;

        $params = array(
            "ISSUE_ID" => $IssueId
        );
        return $this->createAndExecuteQueryAndBuildResponseData(InstanceProvider::getEntityManager(), $query,$params);
    }

    public function operadorByResponse($iddepacaoope,$cdgroperresp,$cdCliente) {
        if($iddepacaoope == 'A'){
            $query =
                <<<SQL
               SELECT
               OP.CDOPERADOR,
               OP.NMOPERADOR
            FROM
               OPERGRUPOP OG,
               OPERADOR OP
            WHERE
               OG.CDGRUPOPER = :P_CDGRUPOPER AND
               OG.CDOPERADOR = OP.CDOPERADOR AND
               OP.IDOPERATIVO = :P_IDDEPACAOOPE
            ORDER BY OP.NMOPERADOR
SQL;

        }else{
            $query =
                <<<SQL
               SELECT DISTINCT
                   OC.CDOPERADOR,
                   OC.NRPRIOPERCLI,
                   O.NMOPERADOR,
                   O.DSEMAILOPER
                FROM
                   OPERCLIE OC,
                   OPERADOR O
                WHERE
                   OC.CDCLIENTE = :P_CDCLIENTE  AND
                   O.CDOPERADOR = OC.CDOPERADOR AND
                   IDTPOPERCLIE = :P_IDDEPACAOOPE
                ORDER BY O.NMOPERADOR
SQL;
        }


        $params = array(
            "P_IDDEPACAOOPE" => $iddepacaoope,
            "P_CDGRUPOPER" => $cdgroperresp,
            "P_CDCLIENTE" => $cdCliente,
        );
        return $this->createAndExecuteQueryAndBuildResponseData(InstanceProvider::getEntityManager(), $query,$params);
    }

    public function operadorGrupoByResponse($cdfasetram,$cdfastramdes, $nrfastramdes) {
        $query =
            <<<SQL
          SELECT
               FD.NRFASTRAMDES,
               FD.CDFASTRAMDES,
               FA.CDGROPERRESP,
               CASE WHEN FA.IDDEPACAOOPE = 'G'
                             THEN 'A'
                             ELSE FA.IDDEPACAOOPE
                         END AS IDDEPACAOOPE
            FROM
               FASETRAMDEST FD,
               FASETRAM FA
            WHERE
               FD.CDFASETRAM   = :P_CDFASETRAM   AND
               FD.CDFASTRAMDES = :P_CDFASTRAMDES AND
               FD.NRFASTRAMDES = :P_NRFASTRAMDES AND
               FD.CDFASTRAMDES = FA.CDFASETRAM
SQL;

        $params = array(
            "P_CDFASTRAMDES" => $cdfastramdes,
            "P_NRFASTRAMDES" => $nrfastramdes,
            "P_CDFASETRAM" => $cdfasetram,
        );
        $result = $this->getEntityManager()->getConnection()->fetchAll($query, $params);
        return count($result) > 0 ? array($result[0]['IDDEPACAOOPE'],$result[0]['CDGROPERRESP']) : array();
    }

    public function findCallTypeRichClient($id,$userId,$situationFilters)
    {

        $gepWaiting = $situationFilters['GEP']['WAITING'];
        $validate = \Util\Utilities::getValidateBudgetFaseGep();
        $refuse = \Util\Utilities::getRefuseBudgetFaseGep();

        $query = <<<SQL
          SELECT  C.NRCALL AS ID, C.NRORG, SER.DSREFESOLI AS REFERENCE,  ASS.CDASSUNTO AS CLASSIFICATION,CLI.CDCLIENTE, 
                        TO_CHAR(C.DTCONPINIT, 'DD/MM/YYYY') AS APPOINTMENT_DATE_INIT,TO_CHAR(C.DTCONPCUREN, 'DD/MM/YYYY') AS APPOINTMENT_DATE,
                  DES.DSSOLISERV AS DESCRIPTION,OPE.CDOPERADOR AS REPORTER_BY,
                  TO_CHAR(C.DTULTATU, 'DD/MM/YYYY') AS UPDATED_AT, SER.CDPRODCOMP AS PRODUCT, C.TYPE,OPE.NMOPERADOR AS REPORTER_NAME, CLI.NMRAZSOCCLIE,
                  C.CALL_RELATIONSHIP, C.CAUSE, C.CDOPERULTATU, C.IDATIVO,C.IDATIVO AS STATUS,
                  SER.CDPRODGRUP AS PRODUCT_TYPE, PRO.NMPRODUTO AS PRODUCT_NAME,PRT.NMPRODUTO as PRODUCT_TYPE_NAME,
                  NVL(CLA.LABEL,ASS.NMASSUNTO) AS CLASSIFICATION_NAME, ASS.IDORIGOBRSOL AS CLASSIFICATION_IDORIGOBRSOL,
                  SER.CDVERPROORIG AS PRODUCT_VERSION,SER.CDVERPROORIG AS PRODUCT_VERSION_LABEL,
                  SER.CDRELVERORIG AS PRODUCT_RELEASE,SER.CDRELVERORIG AS PRODUCT_RELEASE_LABEL,
                  SER.CDPATRELORIG AS PRODUCT_PATCH,SER.CDPATRELORIG AS PRODUCT_PATCH_LABEL,
                  CASE WHEN $gepWaiting
                         THEN 'Y'
                         ELSE 'N'
                     END AS WAITING,
                   CASE WHEN SER.CDFASETRAM IN ('$validate')
                       THEN 'Y'
                       ELSE 'N'
                     END AS ENABLE_APPROVAL,
                   CASE WHEN $gepWaiting AND SER.CDFASETRAM NOT IN (SELECT CDFATRAMENCE FROM PARAMGERAL PARA WHERE PARA.NRORG = C.NRORG)
                         THEN 'Y'
                         ELSE 'N'
                     END AS ENABLE_CLOSED,
                      CASE WHEN EXISTS (  SELECT NRSEQORCAMENTO AS EXTERNAL_ID
                                            FROM FCT_ORCAMENTO
                                           WHERE NRSOLISERV = SER.NRSOLISERV
                                             AND IDATIVO = 'S'
                                             AND IDSITUACAOORCAMENTO <> 'CANCELADO')
                           THEN 'Y'
                           ELSE 'N'
                      END AS IS_INTEGRATION_SM,
                  (
                     SELECT rtrim(xmlagg(xmlelement(e,CLR.LABEL ,',').extract('//text()') order by CLR.LABEL ).GetClobVal(),',')
                     FROM   CP_CALL_LABEL_RELATION CLR
                     JOIN   CP_CALL_LABEL LAB ON(LAB.NRCALLLABEL = CLR.LABEL )
                     WHERE  CLR.CALL = C.NRCALL
                     AND    CLR.NRORG = C.NRORG
                  ) AS CALL_LABELS,
                   (SELECT S.NRSOLISERV
                   FROM SOLISERV S
                   JOIN ORCASOLICITA O ON (O.CDFILIAL = S.CDFILIAL AND O.NRSOLISERV = S.NRSOLISERV)
                  WHERE S.NRSOLISERV = SER.NRSOLISERV
                    AND ROWNUM = 1
                     AND (SELECT CDFASETRAM
                                   FROM ( SELECT CDFASETRAM
                                            FROM LOGFASETRAM LFT
                                           WHERE LFT.NRSOLISERV = SER.NRSOLISERV
                                             AND CDFASETRAM IN ('$refuse','$validate')
                                        ORDER BY DTALFASETRAM DESC)
                                  WHERE ROWNUM = 1) IN ( '$validate' )) AS WITH_BUDGET,C.TEXT_SMALL_1, C.TEXT_SMALL_2, C.TEXT_SMALL_3, C.TEXT_SMALL_4, C.TEXT_SMALL_5,
                                  C.TEXT_LONG_1,C.TEXT_LONG_2,C.TEXT_LONG_3,C.TEXT_LONG_4,C.TEXT_LONG_5,C.NUMBER_1,C.NUMBER_2,C.NUMBER_3,C.NUMBER_4,C.NUMBER_5,
                                  TO_CHAR(DATE_1, 'DD/MM/YYYY') AS DATE_1,TO_CHAR(DATE_2, 'DD/MM/YYYY') AS DATE_2,TO_CHAR(DATE_3, 'DD/MM/YYYY') AS DATE_3,TO_CHAR(DATE_4, 'DD/MM/YYYY') AS DATE_4,TO_CHAR(DATE_5, 'DD/MM/YYYY') AS DATE_5
                FROM CP_CALL C
                JOIN SOLISERV SER ON  (SER.NRSOLISERV = C.CALL_RELATIONSHIP)
                JOIN DESCSOLI DES ON DES.NRSOLISERV = SER.NRSOLISERV
                JOIN OPERADOR OPE ON (OPE.CDOPERADOR = SER.CDOPERADOR )
                JOIN PRODUTO PRO  ON PRO.CDPRODUTO = SER.CDPRODCOMP
                JOIN PRODUTO PRT ON  (PRT.CDPRODUTO = SER.CDPRODGRUP)
           LEFT JOIN ASSUNTO ASS ON ASS.CDASSUNTO = SER.CDASSUNTO
           LEFT JOIN CLIENTE CLI ON CLI.CDCLIENTE = SER.CDCLIENTE
           LEFT JOIN CLASSIFICATION CLA ON (CLA.CDASSUNTO = ASS.CDASSUNTO)
               WHERE C.NRCALL = :CALL_ID
SQL;

        $params = array(
            "CALL_ID" => $id,
            "USER_ID" => $userId,
        );
        $result =  $this->getEntityManager()->getConnection()->fetchAll($query, $params);
        return count($result)>0 ? $result[0] : array();
    }

}
